package com.yandex.syrinxsdk.sample.activity;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentActivity;

import com.yandex.syrinxsdk.sample.R;
import com.yandex.syrinxsdk.sample.fragment.InputFragment;
import com.yandex.syrinxsdk.sample.fragment.MainFragment;
import com.yandex.syrinxsdk.sample.fragment.ResultFragment;
import com.yandex.syrinxsdk.sample.helper.ExtrasHelper;
import com.yandex.syrinxsdk.sample.helper.FragmentHelper;

public class MainActivity extends FragmentActivity implements MainFragment.OnFragmentInteractionListener,
        InputFragment.InputFragmentListener, ResultFragment.OnFragmentInteractionListener {

    private static final int frameLayout = R.id.frame;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        MainFragment mainFragment = MainFragment.newInstance(savedInstanceState);
        FragmentHelper.replace(getSupportFragmentManager(), frameLayout, mainFragment);
    }

    @Override
    public void onStartTransmit() {
        InputFragment inputFragment = InputFragment.newInstance();
        FragmentHelper.replaceAndAddToBackStack(getSupportFragmentManager(), frameLayout, inputFragment);
    }

    @Override
    public void onFinishTransmit(@NonNull String result) {
        Bundle bundle = new Bundle();
        ExtrasHelper.putString(bundle, ResultFragment.ARG_RESULT_TEXT, result);
        ExtrasHelper.putBoolean(bundle, ResultFragment.ARG_WAS_RECEIVED, false);
        ResultFragment resultFragment = ResultFragment.newInstance(bundle);
        FragmentHelper.replace(getSupportFragmentManager(), frameLayout, resultFragment);
    }

    @Override
    public void onCanceledTransmit(@NonNull final String result) {
        Bundle bundle = new Bundle();
        ExtrasHelper.putString(bundle, ResultFragment.ARG_RESULT_TEXT, result);
        ExtrasHelper.putBoolean(bundle, ResultFragment.ARG_WAS_RECEIVED, false);
        ExtrasHelper.putBoolean(bundle, ResultFragment.ARG_WAS_CANCELED, true);
        ResultFragment resultFragment = ResultFragment.newInstance(bundle);
        FragmentHelper.replace(getSupportFragmentManager(), frameLayout, resultFragment);
    }

    public void onFinishReceived(@Nullable String result) {
        Bundle bundle = new Bundle();
        ExtrasHelper.putString(bundle, ResultFragment.ARG_RESULT_TEXT, result);
        ExtrasHelper.putBoolean(bundle, ResultFragment.ARG_WAS_RECEIVED, true);
        ResultFragment resultFragment = ResultFragment.newInstance(bundle);
        FragmentHelper.replace(getSupportFragmentManager(), frameLayout, resultFragment);
    }


    @Override
    public void onStartTransmitPressed(@NonNull String messageToTransmit) {
        Intent intent = new Intent();
        intent.putExtra(MainFragment.ARG_STRING_TO_TRANSMIT, messageToTransmit);
        Fragment fragmentByTag = getSupportFragmentManager().findFragmentByTag(MainFragment.class.getSimpleName());
        fragmentByTag.onActivityResult(MainFragment.REQUEST_CODE, Activity.RESULT_OK, intent);
        getSupportFragmentManager().popBackStack();
    }

    @Override
    public void onDone() {
        MainFragment mainFragment = MainFragment.newInstance(null);
        FragmentHelper.replace(getSupportFragmentManager(), frameLayout, mainFragment);
    }

    @Override
    public void onRepeat(@NonNull String messageToTransmit) {
        Bundle bundle = new Bundle();
        bundle.putString(MainFragment.ARG_STRING_TO_TRANSMIT, messageToTransmit);
        MainFragment fragment = MainFragment.newInstance(bundle);
        FragmentHelper.replace(getSupportFragmentManager(), frameLayout, fragment);
    }

    @Override
    public void onBackPressed() {
        boolean isHandled = false;
        final Fragment topFragment = getSupportFragmentManager().findFragmentById(frameLayout);
        if (topFragment instanceof MainFragment) {
            isHandled = ((MainFragment) topFragment).onBackPressed();
        }

        if (!isHandled) {
            super.onBackPressed();
        }
    }
}
