package com.yandex.syrinxsdk.sample.fragment;

import android.app.Activity;
import android.content.Context;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.v4.app.Fragment;
import android.text.Editable;
import android.text.TextUtils;
import android.text.TextWatcher;
import android.text.util.Linkify;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.inputmethod.EditorInfo;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;

import com.yandex.syrinxsdk.sample.syrinx.SyrinxService;
import com.yandex.syrinxsdk.sample.R;
import com.yandex.syrinxsdk.sample.helper.ExtrasHelper;
import com.yandex.syrinxsdk.sample.helper.FragmentHelper;
import com.yandex.syrinxsdk.sample.helper.KeyboardHelper;

public class InputFragment extends Fragment {
    private static final String ARG_TEXT = "ARG_TEXT";

    @Nullable
    private String textToTransmit;

    @Nullable
    private InputFragmentListener listener;

    @NonNull
    public static InputFragment newInstance() {
        return new InputFragment();
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        textToTransmit = ExtrasHelper.getString(getArguments(), ARG_TEXT);
        FragmentHelper.changeActionBarColor(getActivity(), R.color.light_grey);
    }

    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        return inflater.inflate(R.layout.fragment_input, container, false);
    }

    @Override
    public void onViewCreated(View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);

        // input text
        final EditText inputText = (EditText) view.findViewById(R.id.input_text);

        //transmit btn
        final Button transmitBtn = (Button) view.findViewById(R.id.transmit_btn);
        transmitBtn.setEnabled(!TextUtils.isEmpty(textToTransmit));
        transmitBtn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                onTransmitPressed(inputText);
            }
        });

        inputText.setText(textToTransmit);
        inputText.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {
                //do nothing
            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
                //do nothing
            }

            @Override
            public void afterTextChanged(@NonNull Editable s) {
                Linkify.addLinks(s, Linkify.ALL);
                transmitBtn.setEnabled(s.length() > 0 && s.toString().getBytes().length <=
                        SyrinxService.MAX_BYTES_TO_TRANSMIT);
            }
        });

        inputText.setOnEditorActionListener(new TextView.OnEditorActionListener() {
            @Override
            public boolean onEditorAction(TextView v, int actionId, KeyEvent event) {
                if (actionId == EditorInfo.IME_ACTION_SEND) {
                    if (inputText.getText().length() > 0) {
                        onTransmitPressed(inputText);
                        return true;
                    }
                }
                return false;
            }
        });
        inputText.requestFocus();
    }

    private void onTransmitPressed(@NonNull EditText inputText) {
        String messageToTransmit = inputText.getText().toString();
        KeyboardHelper.hideKeyboard(InputFragment.this.getActivity(), inputText);
        if (listener != null) {
            listener.onStartTransmitPressed(messageToTransmit);
        } else {
            getFragmentManager().popBackStack();
        }
    }

    @Override
    public void onAttach(Context context) {
        super.onAttach(context);
        if (context instanceof Activity) {
            Activity activity = (Activity) context;
            try {
                listener = (InputFragmentListener) activity;
            } catch (ClassCastException e) {
                throw new ClassCastException(activity.toString()
                        + " must implement " + InputFragmentListener.class.getSimpleName());
            }
        }
    }

    @Override
    public void onDetach() {
        super.onDetach();
        listener = null;
    }

    public interface InputFragmentListener {
        void onStartTransmitPressed(@NonNull String messageToTransmit);
    }

}
