package com.yandex.syrinxsdk.sample.fragment;

import android.app.Activity;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.os.Bundle;
import android.os.IBinder;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.v4.app.Fragment;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.TextView;

import com.yandex.syrinxsdk.sample.syrinx.SyrinxBinder;
import com.yandex.syrinxsdk.sample.syrinx.SyrinxListener;
import com.yandex.syrinxsdk.sample.syrinx.SyrinxService;
import com.yandex.syrinxsdk.sample.R;
import com.yandex.syrinxsdk.sample.helper.ExtrasHelper;
import com.yandex.syrinxsdk.sample.helper.FragmentHelper;
import com.yandex.syrinxsdk.sample.helper.MSGHandler;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import lecho.lib.hellocharts.model.Column;
import lecho.lib.hellocharts.model.ColumnChartData;
import lecho.lib.hellocharts.model.SubcolumnValue;
import lecho.lib.hellocharts.view.ColumnChartView;


public class MainFragment extends Fragment implements SyrinxListener, MSGHandler.HandlerListener {
    public static final String ARG_STRING_TO_TRANSMIT = "ARG_STRING_TO_TRANSMIT";

    public static final int REQUEST_CODE = 1;

    public static final int MSG_UPDATE_CHART = 1;

    private static final int STATE_LISTENING = 1;
    private static final int STATE_RECEIVING = 2;
    private static final int STATE_TRANSMITTING = 3;

    private int currentState = STATE_LISTENING;
    @Nullable
    private String stringToTransmit;
    @Nullable
    private OnFragmentInteractionListener listener;

    private Button startTransmitBtn;
    private TextView stateLabel;
    @Nullable
    private SyrinxBinder syrinxBinder;
    private Intent serviceIntent;

    private MSGHandler handler;
    private ColumnChartView chart;
    private ColumnChartData data;
    private static final int numColumns = 150;
    private static final int numSubColumns = 1;
    private int columnColor;
    private int currentMin = 100;
    private final Random random = new Random();

    @NonNull
    private final ServiceConnection connection = new ServiceConnection() {

        @Override
        public void onServiceConnected(ComponentName name, IBinder service) {
            syrinxBinder = (SyrinxBinder) service;
            syrinxBinder.setListener(MainFragment.this);
            if (currentState == STATE_TRANSMITTING) {
                new Thread(new Runnable() {
                    @Override
                    public void run() {
                        syrinxBinder.transmit(stringToTransmit);
                    }
                }).start();
            }
        }

        @Override
        public void onServiceDisconnected(ComponentName name) {
            if (syrinxBinder != null) {
                syrinxBinder.unsubscribeListener();
                syrinxBinder = null;
            }
        }
    };


    @NonNull
    public static MainFragment newInstance(Bundle args) {
        MainFragment fragment = new MainFragment();
        fragment.setArguments(args);
        return fragment;
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        serviceIntent = new Intent(getActivity(), SyrinxService.class);
        handler = new MSGHandler(this);
    }


    @Override
    public void onResume() {
        super.onResume();
        setState();
        getActivity().startService(serviceIntent);
        getActivity().bindService(serviceIntent, connection, Context.BIND_AUTO_CREATE);
        handler.sendEmptyMessage(MSG_UPDATE_CHART);
    }

    private void setState() {
        if (!TextUtils.isEmpty(stringToTransmit)) {
            setState(STATE_TRANSMITTING);
        } else if (getArguments() != null) {
            stringToTransmit = ExtrasHelper.getString(getArguments(), ARG_STRING_TO_TRANSMIT);
            if (!TextUtils.isEmpty(stringToTransmit)) {
                setState(STATE_TRANSMITTING);
            }
        } else {
            setState(STATE_LISTENING);
        }
    }

    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        return inflater.inflate(R.layout.fragment_main, container, false);
    }

    @Override
    public void onViewCreated(View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        stateLabel = (TextView) view.findViewById(R.id.state_label);
        startTransmitBtn = (Button) view.findViewById(R.id.send_button);
        startTransmitBtn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (listener != null) {
                    listener.onStartTransmit();
                }
            }
        });
        setState();
        initChart(view);
    }

    @Override
    public void onAttach(Context context) {
        super.onAttach(context);
        if (context instanceof Activity) {
            Activity activity = (Activity) context;
            try {
                listener = (OnFragmentInteractionListener) activity;
            } catch (ClassCastException e) {
                throw new ClassCastException(activity.toString()
                        + " must implement " + OnFragmentInteractionListener.class.getSimpleName());
            }
        }
        columnColor = getResources().getColor(R.color.white);
    }


    @Override
    public void onDetach() {
        super.onDetach();
        listener = null;
    }

    @Override
    public void onDestroy() {
        getActivity().stopService(serviceIntent);
        super.onDestroy();
    }

    @Override
    public void onReceivingStarted() {
        if (currentState == STATE_LISTENING) {
            setState(STATE_RECEIVING);
        }
    }

    @Override
    public void onReceivingEnded() {
        //do nothing
    }

    @Override
    public void onMessageReceived(@Nullable String message) {
        if (currentState == STATE_RECEIVING) {
            if (listener != null) {
                if (message == null) {
                    message = getResources().getString(R.string.error_message_received);
                }
                listener.onFinishReceived(message);
            }
        }
    }

    @Override
    public void onMessagePartReceived(@Nullable String message) {
        //do nothing
    }

    @Override
    public void onLevelChanged(int level, int minimum, int maximum) {
        //visualize here
        if (currentState == STATE_LISTENING) {
            currentMin = 110;
        } else {
            if (level == 0) {
                currentMin = (minimum + maximum) / 2;
            } else if (level >= 110) {
                currentMin = 90;
            } else {
                currentMin = level;
            }
        }
    }

    @Override
    public void onTransmitEnded(@NonNull String message) {
        if (currentState == STATE_TRANSMITTING) {
            if (listener != null) {
                listener.onFinishTransmit(message);
            }
            stringToTransmit = null;
        }
    }

    @Override
    public void onTransmitCanceled(final String message) {
        if (currentState == STATE_TRANSMITTING) {
            if (listener != null) {
                listener.onCanceledTransmit(message);
            }
            stringToTransmit = null;
        }
    }

    @Override
    public void repeatAction() {
        onAnimateSelected();
    }

    @Override
    public void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
        if (requestCode == REQUEST_CODE && resultCode == Activity.RESULT_OK) {
            if (data != null) {
                String value = data.getStringExtra(ARG_STRING_TO_TRANSMIT);
                if (value != null) {
                    stringToTransmit = value;
                }
            }
        }
    }

    @Override
    public void onPause() {
        super.onPause();
        handler.removeMessages(MSG_UPDATE_CHART);
        if (syrinxBinder != null) {
            syrinxBinder.stop();
        }

        Activity activity = getActivity();
        if (activity != null) {
            activity.unbindService(connection);
        }
    }

    @Override
    public void onDestroyView() {
        super.onDestroyView();
        if (syrinxBinder != null) {
            syrinxBinder.stop();
        }
    }

    public boolean onBackPressed() {
        if (syrinxBinder != null && currentState == STATE_TRANSMITTING) {
            syrinxBinder.stopTransmitting();
            return true;
        }
        return false;
    }

    private void setState(int newState) {
        currentState = newState;
        boolean buttonIsVisible = currentState == STATE_LISTENING;
        if (startTransmitBtn != null) {
            startTransmitBtn.setVisibility(buttonIsVisible ? View.VISIBLE : View.GONE);
        }
        int textResId = -1;
        int columnColorResId = R.color.white;
        int backgroundColorId = R.color.yellow;
        int colorResId = R.color.mustard;
        if (currentState == STATE_LISTENING) {
            textResId = R.string.label_listening;
        } else if (currentState == STATE_RECEIVING) {
            textResId = R.string.label_receiving;
        } else if (currentState == STATE_TRANSMITTING) {
            textResId = R.string.label_transmitting;
            colorResId = R.color.light_grey;
            columnColorResId = R.color.yellow;
            backgroundColorId = R.color.white;
        }

        if (stateLabel != null) {
            stateLabel.setText(textResId);
        }

        View view = getView();
        if (view != null) {
            view.setBackgroundResource(backgroundColorId);
        }
        Activity activity = getActivity();
        if (activity != null) {
            FragmentHelper.changeActionBarColor(activity, colorResId);
        }
        columnColor = getResources().getColor(columnColorResId);
    }


    //chart stuff
    private void initChart(View fragmentView) {
        chart = (ColumnChartView) fragmentView.findViewById(R.id.chart);
        chart.setValueSelectionEnabled(false);
        chart.setValueTouchEnabled(false);
        chart.setScrollEnabled(false);
        chart.setZoomEnabled(false);
        generateDefaultData();
    }

    //chart stuff
    private void prepareDataAnimation() {
        int color = columnColor;
        float min = currentMin;
        for (int i = 0; i < data.getColumns().size(); i++) {
            for (SubcolumnValue value : data.getColumns().get(i).getValues()) {
                float target = random.nextFloat() * (SyrinxService.MAX_LEVEL - min) + min;
                value.setTarget(target);
                value.setColor(color);
            }
        }
    }

    //chart stuff
    private void generateDefaultData() {
        int color = columnColor;
        List<Column> columns = new ArrayList<>();
        List<SubcolumnValue> values;
        float min = currentMin;
        for (int i = 0; i < numColumns; ++i) {
            values = new ArrayList<>();
            for (int j = 0; j < numSubColumns; ++j) {
                values.add(new SubcolumnValue(random.nextFloat() * (SyrinxService.MAX_LEVEL - min) + min, color));
            }

            Column column = new Column(values);
            column.setHasLabels(false);
            column.setHasLabelsOnlyForSelected(false);
            columns.add(column);
        }

        data = new ColumnChartData(columns);
        data.setFillRatio(1);
        data.setStacked(true);
        chart.setColumnChartData(data);
    }

    //chart stuff
    private void onAnimateSelected() {
        prepareDataAnimation();
        chart.startDataAnimation();
    }

    public interface OnFragmentInteractionListener {
        void onStartTransmit();

        void onFinishTransmit(@NonNull String result);

        void onCanceledTransmit(@NonNull String result);

        void onFinishReceived(@Nullable String result);
    }
}
