package com.yandex.syrinxsdk.sample.fragment;

import android.app.Activity;
import android.content.Context;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.v4.app.Fragment;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import com.yandex.syrinxsdk.sample.R;
import com.yandex.syrinxsdk.sample.helper.FragmentHelper;
import com.yandex.syrinxsdk.sample.helper.ExtrasHelper;
import com.yandex.syrinxsdk.sample.helper.IntentHelper;

public class ResultFragment extends Fragment {
    public static final String ARG_RESULT_TEXT = "ARG_RESULT_TEXT";
    public static final String ARG_WAS_RECEIVED = "ARG_WAS_RECEIVED";
    public static final String ARG_WAS_CANCELED = "ARG_WAS_CANCELED";

    private String stringResult;
    private boolean wasReceived;
    private boolean wasCanceled;

    @Nullable
    private OnFragmentInteractionListener listener;

    @NonNull
    public static ResultFragment newInstance(Bundle args) {
        ResultFragment fragment = new ResultFragment();
        fragment.setArguments(args);
        return fragment;
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        stringResult = ExtrasHelper.getString(getArguments(), ARG_RESULT_TEXT);
        wasReceived = ExtrasHelper.getBoolean(getArguments(), ARG_WAS_RECEIVED);
        wasCanceled = ExtrasHelper.getBoolean(getArguments(), ARG_WAS_CANCELED);
    }

    @Override
    public void onResume() {
        super.onResume();
        Activity activity = getActivity();
        if (activity != null) {
            FragmentHelper.changeActionBarColor(activity, R.color.grey);
        }
    }

    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        return inflater.inflate(R.layout.fragment_result, container, false);
    }

    @Override
    public void onViewCreated(View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);

        //action label
        TextView actionLabel = (TextView) view.findViewById(R.id.action_label);
        actionLabel.setText(wasReceived ? R.string.label_received
                : wasCanceled ? R.string.label_canceled : R.string.label_sent);

        //message text
        final TextView sentText = (TextView) view.findViewById(R.id.sent_text);
        sentText.post(new Runnable() {
            @Override
            public void run() {
                sentText.setText(stringResult);
            }
        });

        //share btn
        View shareBtn = view.findViewById(R.id.share_btn);

        //refresh btn
        View refreshBtn = view.findViewById(R.id.refresh_btn);

        if (wasReceived) {
            refreshBtn.setVisibility(View.GONE);
            shareBtn.setVisibility(View.VISIBLE);
            shareBtn.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    IntentHelper.shareText(getActivity(), stringResult);
                }
            });
        } else {
            shareBtn.setVisibility(View.GONE);
            refreshBtn.setVisibility(View.VISIBLE);
            refreshBtn.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    if (listener != null) {
                        listener.onRepeat(stringResult);
                    }
                }
            });
        }

        //done btn
        View doneBtn = view.findViewById(R.id.done_btn);
        doneBtn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (listener != null) {
                    listener.onDone();
                }
            }
        });
    }



    @Override
    public void onAttach(Context context) {
        super.onAttach(context);
        if (context instanceof Activity) {
            Activity activity = (Activity) context;
            try {
                listener = (OnFragmentInteractionListener) activity;
            } catch (ClassCastException e) {
                throw new ClassCastException(activity.toString()
                        + " must implement " + OnFragmentInteractionListener.class.getSimpleName());
            }
        }
    }

    @Override
    public void onDetach() {
        super.onDetach();
        listener = null;
    }

    public interface OnFragmentInteractionListener {

        void onDone();

        void onRepeat(String messageToTransmit);
    }

}
