package com.yandex.syrinxsdk.sample.syrinx;

import android.os.Binder;


public class SyrinxBinder extends Binder {
    private final SyrinxService service;

    public SyrinxBinder(SyrinxService service) {
        this.service = service;
    }

    /**
     * Подписывается листенер на сервис.
     *
     * @param callback листенер
     * @return @false, если листенер уже есть
     */
    public boolean setListener(SyrinxListener callback) {
        return service.subscribe(callback);
    }

    /**
     * Отписывает текущий листенер .
     */
    public void unsubscribeListener() {
        service.unsubscribe();
    }

    /**
     * Передает сообщение. Вызов происходит синхронно.
     *
     * @param message сообщение для передачи.
     */
    public void transmit(String message) {
        service.transmit(message);
    }

    /**
     * Останавливает передачу сообщения.
     */
    public void stopTransmitting() {
        service.stopTransmitting();
    }

    /**
     * Проверяет передается ли в данный момент сообщение.
     *
     * @return @true если в данный момент идет передача сообщения, @false в противном случае.
     */
    public boolean isTransmitting() {
        return service.isTransmitting();
    }


    /**
     * Останавливает сервис.
     */
    public void stop() {
        service.stopSelf();
    }

    /**
     * Останавливает прослушивание.
     */
    public void stopListening() {
        service.stopListening();
    }
}
