package com.yandex.syrinxsdk.sample.syrinx;


/**
 * Callback-интерфейс
 */
public interface SyrinxListener{

    /**
     * Вызывается, когда сервис начинает прием сообщения.
     */
    void onReceivingStarted();

    /**
     * Взывается, когда сервис закончил прием сообщения.
     */
    void onReceivingEnded();

    /**
     * Вызывается, когда сервис принял сообщение.
     *
     * @param message принятое сообщение. Если message равно @null, сообщение было принято с ошибкой.
     */
    void onMessageReceived(String message);

    /**
     * Вызывается, когда принята часть сообщения. Важно: принятый кусов может
     * содержать ошибки, которых не будет в итоговом принятом сообщении.
     *
     * @param message частично полученное сообщение
     */
    void onMessagePartReceived(String message);

    /**
     * Сообщает уровень сигнала от микрофона в дицибелах.
     *
     * @param level   текущий уровень сигнла
     * @param minimum минимальный уровень сигнала
     * @param maximum максимальный уровень сигнала
     */
    void onLevelChanged(int level, int minimum, int maximum);

    /**
     * Вызывается, когда сообщение полностью отправлено
     *
     * @param message отправленное сообщение.
     */
    void onTransmitEnded(String message);

    /**
     * Вызывается, когда передача сообщения прервана.
     *
     * @param message отправляемое сообщение.
     */
    void onTransmitCanceled(String message);
}
